package Text::Xslate::EscapedString;
# the package is implemented intentionally in Text::Xslate
use Text::Xslate;
1;
__END__

=head1 NAME

Text::Xslate::EscapedString - The escaped string representation

=head1 DESCRIPTION

EscapedString is strings marked as escaped so that Xslate does not escape them.

Note that you cannot extend this class.

=cut
